@@ -36,6 +36,8 @@
 #include "ac3.h"
 #include "../aclib/ac.h"
 
+extern pthread_mutex_t init_avcodec_lock;
+
 static AVCodec        *mpa_codec = NULL;
 static AVCodecContext mpa_ctx;
 static char           *mpa_buf     = NULL;
@@ -70,7 +72,11 @@ static int            mpa_bytes_ps, mpa_bytes_pf;
 
 static int verbose=TC_DEBUG;
 #define IS_AUDIO_MONO	(avi_aud_chan == 1)
+#ifdef LAME_3_89
 #define IS_VBR		(lame_get_VBR(lgf) != vbr_off)
+#else
+#define IS_VBR          0
+#endif
 /* Output buffer */
 #define OUTPUT_SIZE	SIZE_PCM_FRAME	
 static char *output   = NULL;
@@ -137,7 +143,7 @@ static void error(const char *s, ...)
 	va_start(ap, s);
 	vfprintf(stderr, s, ap);
 	va_end(ap);
-	putchar('\n');
+	fputs("\n", stderr);
 	
 	return;
 }
@@ -156,7 +162,7 @@ static void debug(const char *s, ...)
 		va_start(ap, s);
 		vfprintf(stderr, s, ap);
 		va_end(ap);
-		putchar('\n');
+		fputs("\n", stderr);
 	}
 	
 	return;
@@ -378,9 +384,11 @@ static int audio_init_ffmpeg(vob_t *vob, int o_codec)
 
     // INIT
 
+    pthread_mutex_lock(&init_avcodec_lock);
     avcodec_init();
     register_avcodec(&ac3_encoder);
     register_avcodec(&mp2_encoder);
+    pthread_mutex_unlock(&init_avcodec_lock);
 
     switch (o_codec) {
 	case   0x50: codeid = CODEC_ID_MP2; break;
@@ -402,7 +410,7 @@ static int audio_init_ffmpeg(vob_t *vob, int o_codec)
       //--------------------------------------------------------
       memset(&mpa_ctx, 0, sizeof(mpa_ctx));       // default all
       mpa_ctx.bit_rate = vob->mp3bitrate * 1000;  // bitrate dest.
-      mpa_ctx.channels = vob->a_chan;             // channels
+      mpa_ctx.channels = vob->dm_chan;             // channels
       mpa_ctx.sample_rate = vob->a_rate;        
 
       // no resampling currently available, use -J resample
@@ -412,7 +420,7 @@ static int audio_init_ffmpeg(vob_t *vob, int o_codec)
       else { 
 	//ThOe added ffmpeg re-sampling capability
         mpa_ctx.sample_rate = vob->mp3frequency;
-	ReSamplectx = audio_resample_init(vob->a_chan, vob->a_chan,
+	ReSamplectx = audio_resample_init(vob->dm_chan, vob->dm_chan,
 					  vob->mp3frequency, vob->a_rate);
       }
       */
@@ -426,7 +434,7 @@ static int audio_init_ffmpeg(vob_t *vob, int o_codec)
       }
     
       //-- bytes per sample and bytes per frame --
-      mpa_bytes_ps = mpa_ctx.channels * vob->a_bits/8;
+      mpa_bytes_ps = mpa_ctx.channels * vob->dm_bits/8;
       mpa_bytes_pf = mpa_ctx.frame_size * mpa_bytes_ps;
     
       //-- create buffer to hold 1 frame --
@@ -620,7 +628,11 @@ int audio_init(vob_t *vob, int v)
 
 		case CODEC_AC3:
 			debug("AC3->AC3");
-			audio_encode_function = audio_passthrough_ac3;
+			if (vob->out_flag) {
+				audio_encode_function = audio_pass_through;
+			} else {
+				audio_encode_function = audio_passthrough_ac3;
+			}
 			/*
 			 *the bitrate can only be determined in the encoder
 			 * section. `bitrate_flags' will be set to 1 after
@@ -743,7 +755,7 @@ static int audio_write(char *buffer, size_t size, avi_t *avifile)
 	{
 		if (fwrite(buffer, size, 1, fd) != 1)
 		{
-			error("Audio file write error (errno=%d).", errno);
+			error("Audio file write error (errno=%d) [%s].", errno, strerror(errno));
 			return(TC_EXPORT_ERROR);
 		}
 	} else {
@@ -911,8 +923,10 @@ static int audio_encode_ffmpeg(char *aud_buffer, int aud_size, avi_t *avifile)
 	
 	memcpy(&mpa_buf[mpa_buf_ptr], in_buf, bytes_needed);
 	
+	pthread_mutex_lock(&init_avcodec_lock);
 	out_size = avcodec_encode_audio(&mpa_ctx, (unsigned char *)output, 
 					OUTPUT_SIZE, (short *)mpa_buf);
+	pthread_mutex_unlock(&init_avcodec_lock);
 	audio_write(output, out_size, avifile);
 	
         in_size -= bytes_needed; 
