@@ -28,7 +28,7 @@
 #include "vid_aux.h"
 
 #define MOD_NAME    "export_dvraw.so"
-#define MOD_VERSION "v0.3 (2003-07-24)"
+#define MOD_VERSION "v0.4 (2003-10-14)"
 #define MOD_CODEC   "(video) Digital Video | (audio) PCM"
 
 #define MOD_PRE dvraw
@@ -38,7 +38,7 @@ extern int _dv_raw_insert_audio(unsigned char * frame_buf,
                      dv_enc_audio_info_t * audio, int isPAL);
 
 static int verbose_flag=TC_QUIET;
-static int capability_flag=TC_CAP_PCM|TC_CAP_RGB|TC_CAP_YUV|TC_CAP_VID;
+static int capability_flag=TC_CAP_PCM|TC_CAP_RGB|TC_CAP_YUV|TC_CAP_VID|TC_CAP_YUV422;
 
 static int fd;
 
@@ -56,6 +56,7 @@ static int pass_through=0;
 
 static int chans, rate;
 static int dv_yuy2_mode=0;
+static int dv_uyvy_mode=0;
 
 static dv_enc_audio_info_t audio;
 
@@ -99,6 +100,7 @@ static int p_write (int fd, char *buf, size_t len)
    return r;
 }
 
+#if 0
 static void pcm_swap(char *buffer, int len)
 {
   char *in, *out;
@@ -120,7 +122,7 @@ static void pcm_swap(char *buffer, int len)
     out = out+2;
   }
 }
-
+#endif
 
 /* ------------------------------------------------------------ 
  *
@@ -142,6 +144,11 @@ MOD_init
       tmp_buf = bufalloc(PAL_W*PAL_H*2); //max frame
       dv_yuy2_mode=1;
     }
+
+    if (vob->im_v_codec == CODEC_YUV422) {
+      tmp_buf = bufalloc(PAL_W*PAL_H*2); //max frame
+      dv_uyvy_mode=1;
+    }
     
 #ifdef LIBDV_095
     encoder = dv_encoder_new(FALSE, FALSE, FALSE);
@@ -201,6 +208,13 @@ MOD_open
       
       if(verbose & TC_DEBUG) fprintf(stderr, "[%s] raw format is YV12\n", MOD_NAME);
       break;
+
+    case CODEC_YUV422:
+      format=1;
+      
+      if(verbose & TC_DEBUG) fprintf(stderr, "[%s] raw format is UYVY\n", MOD_NAME);
+      break;
+      
       
     case CODEC_RAW:
     case CODEC_RAW_YUV:
@@ -223,8 +237,9 @@ MOD_open
 
     
 #ifdef LIBDV_095
+    // Store aspect ratio - ex_asr uses the value 3 for 16x9
+    encoder->is16x9 = ((vob->ex_asr<0) ? vob->im_asr:vob->ex_asr) == 3;
     encoder->isPAL = (vob->ex_v_height==PAL_H);
-    encoder->is16x9 = FALSE;
     encoder->vlc_encode_passes = 3;
     encoder->static_qno = 0;
     encoder->force_dct = DV_DCT_AUTO;
@@ -237,14 +252,12 @@ MOD_open
   
   if(param->flag == TC_AUDIO) {
     
-      /*
     if (!encoder) {
-      tc_warning("[export_dvraw] -y XXX,dvraw is not possible without the video");
-      tc_warning("[export_dvraw] export module also being dvraw");
+      tc_warn("[export_dvraw] -y XXX,dvraw is not possible without the video");
+      tc_warn("[export_dvraw] export module also being dvraw");
       return (TC_EXPORT_ERROR);
     }
-    */
-    chans = (vob->dm_chan != vob->a_chan) ? vob->dm_chan : vob->a_chan;
+    chans = vob->dm_chan;
     //re-sampling only with -J resample possible
     rate = vob->a_rate;
 
@@ -275,7 +288,8 @@ MOD_open
 
 MOD_encode
 {
-  
+  int i;
+
   if(param->flag == TC_VIDEO) { 
     
     if(pass_through) {
@@ -309,43 +323,48 @@ MOD_encode
 	pixels[1]=(char *) vbuf + (NTSC_W*NTSC_H*5)/4;
       }
       
-      if(dv_yuy2_mode) {	
+      if(dv_yuy2_mode && !dv_uyvy_mode) {	
 	yv12toyuy2(pixels[0], pixels[1], pixels[2], tmp_buf, PAL_W, (encoder->isPAL)? PAL_H : NTSC_H);
 	pixels[0]=tmp_buf;
       }
+
+      if (dv_uyvy_mode) {
+	  uyvytoyuy2(pixels[0], tmp_buf, PAL_W, (encoder->isPAL)? PAL_H : NTSC_H);
+	  pixels[0]=tmp_buf;
+      }
       
       dv_encode_full_frame(encoder, pixels, (format)?e_dv_color_yuv:e_dv_color_rgb, target);
       
-      dv_encode_metadata(target, encoder->isPAL, encoder->is16x9, &now, 0);
-      dv_encode_timecode(target, encoder->isPAL, 0);
-      
-      pcm_swap(param->buffer, param->size);
-
-      // demux audio channels
-      //j=0;
-      //for(i=0; i <param->size/4 ; i++) {
-      //for(ch=0; ch < chans; ch++) {
-      //  audio_bufs[ch][i] = (int16_t) param->buffer[j];
-      //  ++j; 
-      //} 
-      //}
-      
+    }//no pass-through
 #ifdef LIBDV_099
       encoder->samples_this_frame=param->size;
 #endif
+      dv_encode_metadata(target, encoder->isPAL, encoder->is16x9, &now, 0);
+      dv_encode_timecode(target, encoder->isPAL, 0);
 
-      //memcpy(audio.data, param->buffer, param->size);
-      //_dv_raw_insert_audio(target, &audio, encoder->isPAL);
-      
-      //FIXME: need to switch to "dv_encode_full_audio" only
-      audio_bufs[0] = (uint16_t *)param->buffer;
-      dv_encode_full_audio(encoder, audio_bufs, chans, rate, target);
+      // Although dv_encode_full_audio supports 4 channels, the internal
+      // PCM data (param->buffer) is only carrying 2 channels so only deal
+      // with 1 or 2 channel audio.
+      // Work around apparent bug in dv_encode_full_audio when chans == 1
+      // by putting silence in 2nd channel and calling with chans = 2
+      if (chans == 1) {
+          audio_bufs[0] = (int16_t *)param->buffer;
+          memset(audio_bufs[1], 0, DV_AUDIO_MAX_SAMPLES * 2);
+          dv_encode_full_audio(encoder, audio_bufs, 2, rate, target);
+      }
+      else {
+          // assume 2 channel, demultiplex for libdv API
+          for(i=0; i < param->size/4; i++) {
+              audio_bufs[0][i] = ((int16_t *)param->buffer)[i*2];
+              audio_bufs[1][i] = ((int16_t *)param->buffer)[i*2+1];
+          }
+          dv_encode_full_audio(encoder, audio_bufs, chans, rate, target);
+      }
 
 #else    
       //merge audio     
       dvenc_frame(vbuf, param->buffer, param->size, target);
 #endif
-    }//no pass-through
     
     //write raw DV frame
     
