@@ -34,6 +34,8 @@
 
 unsigned char asfhdrguid[16]={0x30,0x26,0xB2,0x75,0x8E,0x66,0xCF,0x11,0xA6,0xD9,0x00,0xAA,0x00,0x62,0xCE,0x6C};
 
+unsigned char mxfmagic[]={0x06,0x0e,0x2b,0x34,0x02,0x05,0x01,0x01};
+
 unsigned char lavheader[14]="LAV Edit List\n";
 
 unsigned char zero_pad[4]={0,0,0,0};
@@ -224,6 +226,13 @@ long fileinfo(int fdes, int skip)
     goto exit;
   }
 
+  // SGI image
+
+  if (cmp_16_bits(buf, TC_MAGIC_SGI)) {
+    id = TC_MAGIC_SGI;
+    goto exit;
+  }
+
   // PPM image
   
   if (strncmp (buf, "P6", 2)==0) {
@@ -238,6 +247,13 @@ long fileinfo(int fdes, int skip)
       goto exit;
   }
 
+  // SGI image
+  
+  if (cmp_16_bits(buf, TC_MAGIC_SGI)) {
+      id = TC_MAGIC_SGI;
+      goto exit;
+  }
+
 
   // transport stream
 
@@ -355,6 +371,13 @@ long fileinfo(int fdes, int skip)
     id = TC_MAGIC_ID3;
     goto exit;
   }
+
+  // iTunes sets an ID3 header that way at the beginning. We search for an
+  // syncword first so it should just work.
+  if (buf[0] == 'I' && buf[1] == 'D' && buf[2] == '3' && buf[3] == 0x02) {
+    id = TC_MAGIC_MP3;
+    goto exit;
+  }
   
   /* -------------------------------------------------------------------
    *
@@ -439,6 +462,10 @@ long fileinfo(int fdes, int skip)
     id = TC_MAGIC_JPEG;
     goto exit;
   }
+  if (cmp_16_bits(buf, 0xFFD8)) {
+    id = TC_MAGIC_JPEG;
+    goto exit;
+  }
   
   // WAVE
   if(strncasecmp(buf  ,"RIFF",4) ==0 &&
@@ -475,6 +502,12 @@ long fileinfo(int fdes, int skip)
     goto exit;
   }
 
+  //MXF
+  if(memcmp(mxfmagic,buf,sizeof(mxfmagic))==0) {
+    id = TC_MAGIC_MXF;
+    goto exit;
+  }
+
   /* -------------------------------------------------------------------
    *
    * more tests
@@ -517,10 +550,11 @@ long streaminfo(int fdes)
   
   // assume this is a valid file descriptor
  
-  int bytes=16;
+  int bytes=16, ret=0;
 
-  if(p_read(fdes, buf, bytes)<bytes) {
-    perror("stream read error");
+  if( (ret = p_read(fdes, buf, bytes))<bytes) {
+    if (ret) fprintf(stderr, "File too short (must be 16 bytes at least)\n");
+    else perror("stream read error");
     return(TC_MAGIC_ERROR);
   }
   
@@ -707,6 +741,12 @@ long streaminfo(int fdes)
     goto exit;
   }
 
+  //MXF
+  if(memcmp(mxfmagic,buf,sizeof(mxfmagic))==0) {
+    id = TC_MAGIC_MXF;
+    goto exit;
+  }
+
   /* -------------------------------------------------------------------
    *
    * exit
@@ -740,9 +780,11 @@ char *filetype(long magic)
   case TC_MAGIC_GIF:          return("GIF image");
   case TC_MAGIC_PPM:          return("PPM image");
   case TC_MAGIC_PGM:          return("PGM image");
+  case TC_MAGIC_SGI:          return("SGI image");
   case TC_MAGIC_RMF:          return("Real Media");
-  case TC_MAGIC_XML:          return("XML file, need to and analize the content");
+  case TC_MAGIC_XML:          return("XML file, need to analyze the content");
   case TC_MAGIC_LAV:          return("LAV Edit List");
+  case TC_MAGIC_MXF:          return("The Material eXchange Format");
   case TC_MAGIC_OGG:          return("OGG Multimedia Container");
 
   case TC_MAGIC_RAW:          return("RAW stream");
