@@ -36,7 +36,7 @@
 #include "import_def.h"
 
 static int verbose_flag=TC_QUIET;
-static int capability_flag=TC_CAP_PCM|TC_CAP_RGB|TC_CAP_AUD|TC_CAP_VID;
+static int capability_flag=TC_CAP_PCM|TC_CAP_RGB|TC_CAP_AUD|TC_CAP_VID|TC_CAP_YUV422;
 
 static avi_t *avifile1=NULL;
 static avi_t *avifile2=NULL;
