@@ -33,6 +33,7 @@
 #define TC_CODEC_AC3                   0x00002000
 #define TC_CODEC_A52                   0x00002001
 #define TC_CODEC_YV12                  0x32315659
+#define TC_CODEC_UYVY                  0x59565955
 #define TC_CODEC_YUV2                  0x32565559
 #define TC_CODEC_YUY2                  0x32595559
 #define TC_CODEC_M2V                   0x000001b3
@@ -88,6 +89,7 @@
 #define TC_MAGIC_OGG                   0x00000024
 #define TC_MAGIC_AF6                   0x00000025
 #define TC_MAGIC_VNC                   0x00000026
+#define TC_MAGIC_MXF                   0x00000027
 
 #define TC_MAGIC_VOB                   0x000001ba
 #define TC_MAGIC_SOCKET                0xFF00FF00
@@ -118,6 +120,7 @@
 #define TC_MAGIC_TIFF2                 0x00004949
 #define TC_MAGIC_JPEG                  0xFFD8FFE0
 #define TC_MAGIC_BMP                   0x0000424D
+#define TC_MAGIC_SGI                   0x000001DA
 #define TC_MAGIC_PNG                   0x89504e47
 #define TC_MAGIC_GIF                   0x00474946
 #define TC_MAGIC_PPM                   0x00005036
