@@ -70,6 +70,7 @@ void usage(int status)
   fprintf(stderr,"\t -A n              A52 decoder flag [0]\n");
   fprintf(stderr,"\t -C s,e            decode only from start to end ((V) frames/(A) bytes) [all]\n");
   fprintf(stderr,"\t -Y                use libdv YUY2 decoder mode\n");
+  fprintf(stderr,"\t -z r              convert zero padding to silence\n");
   fprintf(stderr,"\t -v                print version\n");
 
   exit(status);
@@ -104,7 +105,7 @@ int main(int argc, char *argv[])
     ipipe.frame_limit[0]=0; 
     ipipe.frame_limit[1]=LONG_MAX; 
 
-    while ((ch = getopt(argc, argv, "Q:t:d:x:i:a:g:vy:s:YC:A:?h")) != -1) {
+    while ((ch = getopt(argc, argv, "Q:t:d:x:i:a:g:vy:s:YC:A:z:?h")) != -1) {
 	
 	switch (ch) {
 	    
@@ -182,6 +183,12 @@ int main(int argc, char *argv[])
 		usage(EXIT_FAILURE);
 	  }
 	  break;
+
+	case 'z':
+	  
+	  if(optarg[0]=='-') usage(EXIT_FAILURE);
+	  ipipe.padrate = atoi(optarg);
+	  break;
 	  
 	case 'h':
 	  usage(EXIT_SUCCESS);
