@@ -253,6 +253,9 @@ int export_init(vob_t *vob, char *a_mod, char *v_mod)
     case CODEC_YUV: 
       cc=(export_para.flag & TC_CAP_YUV);
       break;
+    case CODEC_YUV422: 
+      cc=(export_para.flag & TC_CAP_YUV422);
+      break;
     case CODEC_RAW: 
     case CODEC_RAW_YUV: 
       cc=(export_para.flag & TC_CAP_VID);
@@ -497,6 +500,8 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	return;
       }
 
+      tc_pause();
+
       if(have_vframe_threads==0) {
 	pthread_mutex_lock(&vbuffer_im_fill_lock);
 	--vbuffer_im_fill_ctr;
