@@ -244,7 +244,9 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
       D_arg = 0;
   }
 
-  if(verbose & TC_DEBUG) printf("(%s) suggested AV correction -D %d (%d ms) | AV %d ms | %d ms\n", __FILE__, D_arg, (int) ((D_arg*1000)/vob->fps), (int) (1000*pts_diff), D_arg_ms);	
+  if(verbose & TC_INFO) 
+      printf("[%s] (probe) suggested AV correction -D %d (%d ms) | AV %d ms | %d ms\n",
+	  "transcode", D_arg, (int) ((D_arg*1000)/vob->fps), (int) (1000*pts_diff), D_arg_ms);	
   
   // AV sync correction: case (1)
   //
@@ -314,6 +316,8 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
   }
 
   if(info->track[track].bitrate>0) vob->a_stream_bitrate = info->track[track].bitrate;
+
+  if(info->track[track].padrate>0) vob->a_padrate = info->track[track].padrate;
   
   if( !(*flag & TC_PROBE_NO_ACODEC)) {
     if(info->track[track].format>0) vob->fixme_a_codec = info->track[track].format; 
@@ -468,6 +472,7 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
   case TC_MAGIC_BMP:
   case TC_MAGIC_PNG:
   case TC_MAGIC_GIF:
+  case TC_MAGIC_SGI:
       vob->vmod_probed=std_module[_im_];
       preset |= TC_VIDEO;
       break;
@@ -522,7 +527,8 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
       break;
       
   case TC_MAGIC_WAV:
-    vob->amod_probed=std_module[_yuv4mpeg_];
+    // vob->amod_probed=std_module[_yuv4mpeg_];
+    vob->amod_probed=std_module[_raw_];
     preset |= TC_AUDIO;
     break;
 
@@ -668,7 +674,7 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
     // if input is ogg, do not probe video
     if (probe_further_for_codec) {
 	//overwrite pass-through selection!
-	vob->vmod_probed=std_module[_divx_];
+	vob->vmod_probed=std_module[_ffmpeg_];
 	preset |= TC_VIDEO;
 	if(preset & TC_AUDIO) break;
     }
@@ -691,6 +697,23 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
     //audio
     vob->amod_probed=get_audio_module(vob->fixme_a_codec, vob->has_audio);
     preset |= TC_AUDIO;  
+
+    break;
+    
+  case TC_CODEC_UYVY:
+    vob->im_v_codec=CODEC_YUV422;
+
+    //overwrite pass-through selection!
+    vob->vmod_probed=std_module[_raw_];
+    preset |= TC_VIDEO;
+    
+    if(preset & TC_AUDIO) break;
+    
+    //audio
+    vob->amod_probed=get_audio_module(vob->fixme_a_codec, vob->has_audio);
+    preset |= TC_AUDIO;  
+
+    break;
     
   case TC_CODEC_RGB:
 
@@ -786,6 +809,9 @@ char *codec2str(int f)
   case TC_CODEC_LAV:
     return("LAV");
 
+  case TC_CODEC_PCM:
+    return("PCM");
+
   default:
     return("unknown");
   }
@@ -871,6 +897,8 @@ char *mformat2str(int f)
 	return("MP2");
     case TC_MAGIC_OGG:
 	return("OGG stream");
+    case TC_MAGIC_WAV:
+	return ("WAVE");
     case TC_MAGIC_V4L_VIDEO:
     case TC_MAGIC_V4L_AUDIO:
 	return("V4L");
