@@ -54,6 +54,7 @@ typedef unsigned int uint32_t;
 #endif
 
 #include "tc_defaults.h"
+#include "tc_functions.h"
 
 /* ----------------------------
  *
@@ -153,6 +154,8 @@ typedef struct _vob_t {
   int a_bits;
   int a_rate;
 
+  int a_padrate;            // zero padding rate
+
   int im_a_size;            // import total bytes per audio frame 
   int ex_a_size;            // export total bytes per audio frame 
 
@@ -333,6 +336,7 @@ typedef struct _vob_t {
   int accel;
 
   int video_frames_delay; // delay audio by N frames, (video is late)
+  float m2v_requant;    // Requantize Factor for mpeg2 video streams
   
 } vob_t;
 
@@ -349,6 +353,7 @@ typedef struct pcm_s {
   int chan;
   int bits;
   int bitrate;
+  int padrate;         // byterate for zero padding
   
   int format;
   int lang;
@@ -411,6 +416,8 @@ typedef struct _info_t {
   
   int width;      // logical stream parameter
   int height;     // logical stream parameter
+
+  int padrate;    // zero padding rate
   
   long stype;     // specifies stream type for extract thread
   long codec;     // specifies codec for decoder thread
@@ -480,9 +487,11 @@ typedef struct subtitle_header_s {
 void tc_progress(char *string);
 void tc_import_stop_nolock(void);
 void tc_export_stop_nolock(void);
+/* these function are in tc_functions.{ch}
 void tc_error(char *fmt, ...);
 void tc_info(char *fmt, ...);
 void tc_warn(char *fmt, ...);
+*/
 vob_t *tc_get_vob(void); 
 long tc_get_frames_encoded(void);
 long tc_get_frames_dropped(void);
@@ -494,8 +503,10 @@ void tc_update_frames_cloned(long cc);
 void tc_set_force_exit(void);
 int tc_get_force_exit(void);
 
-void tc_outstream_rotate();
-void tc_outstream_rotate_request();
+void tc_outstream_rotate(void);
+void tc_outstream_rotate_request(void);
+void tc_pause(void);
+void tc_pause_request(void);
 
 void tc_adjust_frame_buffer(int height, int width);
 
@@ -573,6 +584,7 @@ extern pid_t tc_probe_pid;
 # define TC_CAP_MP3         64
 # define TC_CAP_YUY2       128
 # define TC_CAP_DV         256
+# define TC_CAP_YUV422     512
 
 # define TC_MODE_DEFAULT            0
 # define TC_MODE_AVI_SPLIT          1
